"""
:Copyright: 2007-2021 Jochen Kupperschmidt
:License: MIT, see LICENSE for details.
"""

from pathlib import Path
from tempfile import mkstemp

import pytest

from chatrelater.serialization import serialize_data_to_file, load_data


def test_load_save_data(tmp_path):
    """Test saving data and then loading it again.

    Also, KeyErrors are expected if required keys are missing.
    """
    # Assure that missing keys raise a ``KeyError``.
    required_keys = frozenset(['nicknames', 'relations', 'directed'])
    for key in required_keys:
        keys = set(required_keys)
        keys.remove(key)
        assert_required_keys(tmp_path, dict.fromkeys(keys), True)
    assert_required_keys(tmp_path, {'nicknames': None}, True)
    assert_required_keys(tmp_path, dict.fromkeys(required_keys), False)

    # Assure the exact data saved will be loaded afterwards.
    data = {
        'nicknames': 123,
        'relations': 456,
        'directed': True,
    }
    filename = generate_tmp_file(tmp_path)
    serialize_data_to_file(data, filename=filename)
    assert load_data(filename) == (123, 456, True)


def generate_tmp_file(tmp_path: Path) -> Path:
    _, tmp_filename = mkstemp(dir=tmp_path)
    return Path(tmp_filename)


def assert_required_keys(tmp_path, data, should_raise=False):
    filename = generate_tmp_file(tmp_path)

    serialize_data_to_file(data, filename=filename)

    if should_raise:
        with pytest.raises(KeyError):
            load_data(filename)
    else:
        load_data(filename)
